@@ -41,6 +41,7 @@
 #include "gedit-utils.h"
 #include "gedit-convert.h"
 #include "gedit-marshal.h"
+#include "gedit-enum-types.h"
 
 /* Those are for the the gedit_document_loader_new() factory */
 #include "gedit-gio-document-loader.h"
@@ -64,6 +65,7 @@ enum
 	PROP_DOCUMENT,
 	PROP_URI,
 	PROP_ENCODING,
+	PROP_NEWLINE_TYPE
 };
 
 static void
@@ -88,6 +90,9 @@ gedit_document_loader_set_property (GObject      *object,
 			g_return_if_fail (loader->encoding == NULL);
 			loader->encoding = g_value_get_boxed (value);
 			break;
+		case PROP_NEWLINE_TYPE:
+			loader->auto_detected_newline_type = g_value_get_enum (value);
+			break;
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 			break;
@@ -113,6 +118,9 @@ gedit_document_loader_get_property (GObject    *object,
 		case PROP_ENCODING:
 			g_value_set_boxed (value, gedit_document_loader_get_encoding (loader));
 			break;
+		case PROP_NEWLINE_TYPE:
+			g_value_set_enum (value, loader->auto_detected_newline_type);
+			break;
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 			break;
@@ -126,15 +134,33 @@ gedit_document_loader_finalize (GObject *object)
 
 	g_free (loader->uri);
 
+	if (loader->info)
+		g_object_unref (loader->info);
+
 	G_OBJECT_CLASS (gedit_document_loader_parent_class)->finalize (object);
 }
 
+static void
+gedit_document_loader_dispose (GObject *object)
+{
+	GeditDocumentLoader *loader = GEDIT_DOCUMENT_LOADER (object);
+
+	if (loader->info != NULL)
+	{
+		g_object_unref (loader->info);
+		loader->info = NULL;
+	}
+
+	G_OBJECT_CLASS (gedit_document_loader_parent_class)->dispose (object);
+}
+
 static void
 gedit_document_loader_class_init (GeditDocumentLoaderClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
 	object_class->finalize = gedit_document_loader_finalize;
+	object_class->dispose = gedit_document_loader_dispose;
 	object_class->get_property = gedit_document_loader_get_property;
 	object_class->set_property = gedit_document_loader_set_property;
 
@@ -167,6 +193,16 @@ gedit_document_loader_class_init (GeditDocumentLoaderClass *klass)
 							     G_PARAM_CONSTRUCT_ONLY |
 							     G_PARAM_STATIC_STRINGS));
 
+	g_object_class_install_property (object_class, PROP_NEWLINE_TYPE,
+	                                 g_param_spec_enum ("newline-type",
+	                                                    "Newline type",
+	                                                    "The accepted types of line ending",
+	                                                    GEDIT_TYPE_DOCUMENT_NEWLINE_TYPE,
+	                                                    GEDIT_DOCUMENT_NEWLINE_TYPE_LF,
+	                                                    G_PARAM_READWRITE |
+	                                                    G_PARAM_STATIC_NAME |
+	                                                    G_PARAM_STATIC_BLURB));
+
 	signals[LOADING] =
 		g_signal_new ("loading",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -184,129 +220,7 @@ static void
 gedit_document_loader_init (GeditDocumentLoader *loader)
 {
 	loader->used = FALSE;
-}
-
-static void
-insert_text_in_document (GeditDocumentLoader *loader,
-			 const gchar         *text,
-			 gint                 len)
-{
-	GeditDocument *doc = loader->document;
-
-	g_return_if_fail (text != NULL);
-
-	gtk_source_buffer_begin_not_undoable_action (GTK_SOURCE_BUFFER (doc));
-
-	/* If the last char is a newline, don't add it to the buffer (otherwise
-	   GtkTextView shows it as an empty line). See bug #324942. */
-	if ((len > 0) && (text[len-1] == '\n'))
-		len--;
-
-	/* Insert text in the buffer */
-	gtk_text_buffer_set_text (GTK_TEXT_BUFFER (doc), text, len);
-
-	gtk_text_buffer_set_modified (GTK_TEXT_BUFFER (doc), FALSE);
-
-	gtk_source_buffer_end_not_undoable_action (GTK_SOURCE_BUFFER (doc));
-}
-
-/* This function is only meant to be called by child classes */
-gboolean
-gedit_document_loader_update_document_contents (GeditDocumentLoader  *loader,
-					        const gchar          *file_contents,
-					        gint                  file_size,
-					        GError              **error)
-{
-	gedit_debug (DEBUG_LOADER);
-
-	g_return_val_if_fail (file_size >= 0, FALSE);
-	g_return_val_if_fail (file_contents != NULL, FALSE);
-
-	/* short-circuit the case where the file is empty */
-	if (file_size == 0)
-	{
-		if (loader->encoding == NULL)
-			loader->auto_detected_encoding = gedit_encoding_get_current ();
-		insert_text_in_document (loader, "", 0);
-		return TRUE;
-	}
-
-	if (loader->encoding == gedit_encoding_get_utf8 ())
-	{
-		if (g_utf8_validate (file_contents, file_size, NULL))
-		{
-			insert_text_in_document (loader,
-						 file_contents,
-						 file_size);
-			return TRUE;
-		}
-		else
-		{
-			g_set_error (error,
-				     G_CONVERT_ERROR,
-				     G_CONVERT_ERROR_ILLEGAL_SEQUENCE,
-				     "The file you are trying to open contains an invalid byte sequence.");
-			return FALSE;
-		}
-	}
-	else
-	{
-		GError *conv_error = NULL;
-		gchar *converted_text = NULL;
-		gsize new_len = file_size;
-
-		if (loader->encoding == NULL)
-		{
-			/* Autodetecting the encoding: first try with the encoding
-			stored in the metadata, if any */
-
-			if (loader->metadata_encoding != NULL)
-			{
-				converted_text = gedit_convert_to_utf8 (
-								file_contents,
-								file_size,
-								&loader->metadata_encoding,
-								&new_len,
-								NULL);
-
-				if (converted_text != NULL)
-					loader->auto_detected_encoding = loader->metadata_encoding;
-			}
-		}
-
-		if (converted_text == NULL)
-		{
-			loader->auto_detected_encoding = loader->encoding;
-
-			converted_text = gedit_convert_to_utf8 (
-							file_contents,
-							file_size,
-							&loader->auto_detected_encoding,
-							&new_len,
-							&conv_error);
-		}
-
-		if (converted_text == NULL)
-		{
-			g_return_val_if_fail (conv_error != NULL, FALSE);
-
-			g_propagate_error (error, conv_error);
-
-			return FALSE;
-		}
-		else
-		{
-			insert_text_in_document (loader,
-						 converted_text,
-						 new_len);
-
-			g_free (converted_text);
-
-			return TRUE;
-		}
-	}
-
-	g_return_val_if_reached (FALSE);
+	loader->auto_detected_newline_type = GEDIT_DOCUMENT_NEWLINE_TYPE_DEFAULT;
 }
 
 void
@@ -335,24 +249,6 @@ gedit_document_loader_loading (GeditDocumentLoader *loader,
 	}
 }
 
-static const GeditEncoding *
-get_metadata_encoding (const gchar *uri)
-{
-	const GeditEncoding *enc;
-	gchar *charset;
-
-	charset = gedit_metadata_manager_get (uri, "encoding");
-
-	if (charset == NULL)
-		return NULL;
-
-	enc = gedit_encoding_get_from_charset (charset);
-
-	g_free (charset);
-
-	return enc;
-}
-
 /* This is a factory method that returns an appopriate loader
  * for the given uri.
  */
@@ -392,9 +288,6 @@ gedit_document_loader_load (GeditDocumentLoader *loader)
 	g_return_if_fail (loader->used == FALSE);
 	loader->used = TRUE;
 
-	if (loader->encoding == NULL)
-		loader->metadata_encoding = get_metadata_encoding (loader->uri);
-
 	GEDIT_DOCUMENT_LOADER_GET_CLASS (loader)->load (loader);
 }
 
@@ -425,32 +318,6 @@ gedit_document_loader_get_uri (GeditDocumentLoader *loader)
 	return loader->uri;
 }
 
-/* it may return NULL, it's up to gedit-document handle it */
-const gchar *
-gedit_document_loader_get_content_type (GeditDocumentLoader *loader)
-{
-	g_return_val_if_fail (GEDIT_IS_DOCUMENT_LOADER (loader), NULL);
-
-	return GEDIT_DOCUMENT_LOADER_GET_CLASS (loader)->get_content_type (loader);
-}
-
-time_t
-gedit_document_loader_get_mtime (GeditDocumentLoader *loader)
-{
-	g_return_val_if_fail (GEDIT_IS_DOCUMENT_LOADER (loader), 0);
-
-	return GEDIT_DOCUMENT_LOADER_GET_CLASS (loader)->get_mtime (loader);
-}
-
-/* Returns 0 if file size is unknown */
-goffset
-gedit_document_loader_get_file_size (GeditDocumentLoader *loader)
-{
-	g_return_val_if_fail (GEDIT_IS_DOCUMENT_LOADER (loader), 0);
-
-	return GEDIT_DOCUMENT_LOADER_GET_CLASS (loader)->get_file_size (loader);
-}
-
 goffset
 gedit_document_loader_get_bytes_read (GeditDocumentLoader *loader)
 {
@@ -459,22 +326,6 @@ gedit_document_loader_get_bytes_read (GeditDocumentLoader *loader)
 	return GEDIT_DOCUMENT_LOADER_GET_CLASS (loader)->get_bytes_read (loader);
 }
 
-/* In the case the loader does not know if the file is readonly, for example 
-   for most remote files, the function returns FALSE, so that we can try writing
-   and if needed handle the error. */
-gboolean
-gedit_document_loader_get_readonly (GeditDocumentLoader *loader)
-{
-	g_return_val_if_fail (GEDIT_IS_DOCUMENT_LOADER (loader), FALSE);
-
-	/* if configuration says the scheme is not writable, do not query the
-	   actual loader object, and return TRUE */
-	if (!gedit_utils_uri_has_writable_scheme (loader->uri))
-		return TRUE;
-
-	return GEDIT_DOCUMENT_LOADER_GET_CLASS (loader)->get_readonly (loader);
-}
-
 const GeditEncoding *
 gedit_document_loader_get_encoding (GeditDocumentLoader *loader)
 {
@@ -488,3 +339,20 @@ gedit_document_loader_get_encoding (GeditDocumentLoader *loader)
 
 	return loader->auto_detected_encoding;
 }
+
+GeditDocumentNewlineType
+gedit_document_loader_get_newline_type (GeditDocumentLoader *loader)
+{
+	g_return_val_if_fail (GEDIT_IS_DOCUMENT_LOADER (loader),
+			      GEDIT_DOCUMENT_NEWLINE_TYPE_LF);
+
+	return loader->auto_detected_newline_type;
+}
+
+GFileInfo *
+gedit_document_loader_get_info (GeditDocumentLoader *loader)
+{
+	g_return_val_if_fail (GEDIT_IS_DOCUMENT_LOADER (loader), NULL);
+
+	return loader->info;
+}
