@@ -66,6 +66,7 @@ enum {
 	PROP_DOCUMENT,
 	PROP_URI,
 	PROP_ENCODING,
+	PROP_NEWLINE_TYPE,
 	PROP_FLAGS
 };
 
@@ -91,6 +92,9 @@ gedit_document_saver_set_property (GObject      *object,
 			g_return_if_fail (saver->encoding == NULL);
 			saver->encoding = g_value_get_boxed (value);
 			break;
+		case PROP_NEWLINE_TYPE:
+			saver->newline_type = g_value_get_enum (value);
+			break;
 		case PROP_FLAGS:
 			saver->flags = g_value_get_flags (value);
 			break;
@@ -119,6 +123,9 @@ gedit_document_saver_get_property (GObject    *object,
 		case PROP_ENCODING:
 			g_value_set_boxed (value, saver->encoding);
 			break;
+		case PROP_NEWLINE_TYPE:
+			g_value_set_enum (value, saver->newline_type);
+			break;
 		case PROP_FLAGS:
 			g_value_set_flags (value, saver->flags);
 			break;
@@ -139,12 +146,27 @@ gedit_document_saver_finalize (GObject *object)
 	G_OBJECT_CLASS (gedit_document_saver_parent_class)->finalize (object);
 }
 
+static void
+gedit_document_saver_dispose (GObject *object)
+{
+	GeditDocumentSaver *saver = GEDIT_DOCUMENT_SAVER (object);
+
+	if (saver->info != NULL)
+	{
+		g_object_unref (saver->info);
+		saver->info = NULL;
+	}
+
+	G_OBJECT_CLASS (gedit_document_saver_parent_class)->dispose (object);
+}
+
 static void 
 gedit_document_saver_class_init (GeditDocumentSaverClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
 	object_class->finalize = gedit_document_saver_finalize;
+	object_class->dispose = gedit_document_saver_dispose;
 	object_class->set_property = gedit_document_saver_set_property;
 	object_class->get_property = gedit_document_saver_get_property;
 
@@ -178,6 +200,18 @@ gedit_document_saver_class_init (GeditDocumentSaverClass *klass)
 							     G_PARAM_CONSTRUCT_ONLY |
 							     G_PARAM_STATIC_STRINGS));
 
+	g_object_class_install_property (object_class,
+					 PROP_NEWLINE_TYPE,
+					 g_param_spec_enum ("newline-type",
+					                    "Newline type",
+					                    "The accepted types of line ending",
+					                    GEDIT_TYPE_DOCUMENT_NEWLINE_TYPE,
+					                    GEDIT_DOCUMENT_NEWLINE_TYPE_LF,
+					                    G_PARAM_READWRITE |
+					                    G_PARAM_STATIC_NAME |
+					                    G_PARAM_STATIC_BLURB |
+					                    G_PARAM_CONSTRUCT_ONLY));
+
 	g_object_class_install_property (object_class,
 					 PROP_FLAGS,
 					 g_param_spec_flags ("flags",
@@ -208,22 +242,18 @@ gedit_document_saver_init (GeditDocumentSaver *saver)
 }
 
 GeditDocumentSaver *
-gedit_document_saver_new (GeditDocument       *doc,
-			  const gchar         *uri,
-			  const GeditEncoding *encoding,
-			  GeditDocumentSaveFlags flags)
+gedit_document_saver_new (GeditDocument           *doc,
+			  const gchar             *uri,
+			  const GeditEncoding     *encoding,
+			  GeditDocumentNewlineType newline_type,
+			  GeditDocumentSaveFlags   flags)
 {
 	GeditDocumentSaver *saver;
 	GType saver_type;
 
 	g_return_val_if_fail (GEDIT_IS_DOCUMENT (doc), NULL);
 
-#ifndef G_OS_WIN32
-	if (gedit_utils_uri_has_file_scheme (uri))
-		saver_type = GEDIT_TYPE_LOCAL_DOCUMENT_SAVER;
-	else
-#endif
-		saver_type = GEDIT_TYPE_GIO_DOCUMENT_SAVER;
+	saver_type = GEDIT_TYPE_GIO_DOCUMENT_SAVER;
 
 	if (encoding == NULL)
 		encoding = gedit_encoding_get_utf8 ();
@@ -232,6 +262,7 @@ gedit_document_saver_new (GeditDocument       *doc,
 						    "document", doc,
 						    "uri", uri,
 						    "encoding", encoding,
+						    "newline_type", newline_type,
 						    "flags", flags,
 						    NULL));
 
@@ -441,7 +472,7 @@ gedit_document_saver_saving (GeditDocumentSaver *saver,
 
 void
 gedit_document_saver_save (GeditDocumentSaver     *saver,
-			   time_t                  old_mtime)
+			   GTimeVal               *old_mtime)
 {
 	gedit_debug (DEBUG_SAVER);
 
@@ -487,22 +518,6 @@ gedit_document_saver_get_uri (GeditDocumentSaver *saver)
 	return saver->uri;
 }
 
-const gchar *
-gedit_document_saver_get_content_type (GeditDocumentSaver *saver)
-{
-	g_return_val_if_fail (GEDIT_IS_DOCUMENT_SAVER (saver), NULL);
-
-	return GEDIT_DOCUMENT_SAVER_GET_CLASS (saver)->get_content_type (saver);
-}
-
-time_t
-gedit_document_saver_get_mtime (GeditDocumentSaver *saver)
-{
-	g_return_val_if_fail (GEDIT_IS_DOCUMENT_SAVER (saver), 0);
-
-	return GEDIT_DOCUMENT_SAVER_GET_CLASS (saver)->get_mtime (saver);
-}
-
 /* Returns 0 if file size is unknown */
 goffset
 gedit_document_saver_get_file_size (GeditDocumentSaver *saver)
@@ -519,3 +534,11 @@ gedit_document_saver_get_bytes_written (GeditDocumentSaver *saver)
 
 	return GEDIT_DOCUMENT_SAVER_GET_CLASS (saver)->get_bytes_written (saver);
 }
+
+GFileInfo *
+gedit_document_saver_get_info (GeditDocumentSaver *saver)
+{
+	g_return_val_if_fail (GEDIT_IS_DOCUMENT_SAVER (saver), NULL);
+
+	return saver->info;
+}
