@@ -25,7 +25,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-io-error-message-area.c 6901 2009-02-16 17:41:31Z pborelli $
+ * $Id$
  */
  
 /*
@@ -46,9 +46,12 @@
 #include "gedit-convert.h"
 #include "gedit-document.h"
 #include "gedit-io-error-message-area.h"
-#include "gedit-message-area.h"
 #include "gedit-prefs-manager.h"
-#include <gedit/gedit-encodings-option-menu.h>
+#include <gedit/gedit-encodings-combo-box.h>
+
+#if !GTK_CHECK_VERSION (2, 17, 1)
+#include "gedit-message-area.h"
+#endif
 
 #define MAX_URI_IN_DIALOG_LENGTH 50
 
@@ -82,10 +85,43 @@ is_gio_error (const GError *error,
 }
 
 static void
-set_message_area_text_and_icon (GeditMessageArea *message_area,
-				const gchar      *icon_stock_id,
-				const gchar      *primary_text,
-				const gchar      *secondary_text)
+set_contents (GtkWidget *area,
+	      GtkWidget *contents)
+{
+#if !GTK_CHECK_VERSION (2, 17, 1)
+	gedit_message_area_set_contents (GEDIT_MESSAGE_AREA (area),
+					 contents);
+#else
+	GtkWidget *content_area;
+	
+	content_area = gtk_info_bar_get_content_area (GTK_INFO_BAR (area));
+	gtk_container_add (GTK_CONTAINER (content_area), contents);
+#endif
+}
+
+#if GTK_CHECK_VERSION (2, 17, 1)
+static void
+info_bar_add_stock_button_with_text (GtkInfoBar  *infobar,
+				     const gchar *text,
+				     const gchar *stock_id,
+				     gint         response_id)
+{
+	GtkWidget *button;
+	
+	button = gedit_gtk_button_new_with_stock_icon (text, stock_id);
+	gtk_widget_show (button);
+
+	gtk_info_bar_add_action_widget (infobar,
+					button,
+					response_id);
+}
+#endif
+
+static void
+set_message_area_text_and_icon (GtkWidget   *message_area,
+				const gchar *icon_stock_id,
+				const gchar *primary_text,
+				const gchar *secondary_text)
 {
 	GtkWidget *hbox_content;
 	GtkWidget *image;
@@ -96,21 +132,17 @@ set_message_area_text_and_icon (GeditMessageArea *message_area,
 	GtkWidget *secondary_label;
 
 	hbox_content = gtk_hbox_new (FALSE, 8);
-	gtk_widget_show (hbox_content);
 
 	image = gtk_image_new_from_stock (icon_stock_id, GTK_ICON_SIZE_DIALOG);
-	gtk_widget_show (image);
 	gtk_box_pack_start (GTK_BOX (hbox_content), image, FALSE, FALSE, 0);
 	gtk_misc_set_alignment (GTK_MISC (image), 0.5, 0);
 
 	vbox = gtk_vbox_new (FALSE, 6);
-	gtk_widget_show (vbox);
 	gtk_box_pack_start (GTK_BOX (hbox_content), vbox, TRUE, TRUE, 0);
 
 	primary_markup = g_strdup_printf ("<b>%s</b>", primary_text);
 	primary_label = gtk_label_new (primary_markup);
 	g_free (primary_markup);
-	gtk_widget_show (primary_label);
 	gtk_box_pack_start (GTK_BOX (vbox), primary_label, TRUE, TRUE, 0);
 	gtk_label_set_use_markup (GTK_LABEL (primary_label), TRUE);
 	gtk_label_set_line_wrap (GTK_LABEL (primary_label), TRUE);
@@ -118,13 +150,12 @@ set_message_area_text_and_icon (GeditMessageArea *message_area,
 	GTK_WIDGET_SET_FLAGS (primary_label, GTK_CAN_FOCUS);
 	gtk_label_set_selectable (GTK_LABEL (primary_label), TRUE);
 
-  	if (secondary_text != NULL)
-  	{
-  		secondary_markup = g_strdup_printf ("<small>%s</small>",
-  						    secondary_text);
+	if (secondary_text != NULL)
+	{
+		secondary_markup = g_strdup_printf ("<small>%s</small>",
+						    secondary_text);
 		secondary_label = gtk_label_new (secondary_markup);
 		g_free (secondary_markup);
-		gtk_widget_show (secondary_label);
 		gtk_box_pack_start (GTK_BOX (vbox), secondary_label, TRUE, TRUE, 0);
 		GTK_WIDGET_SET_FLAGS (secondary_label, GTK_CAN_FOCUS);
 		gtk_label_set_use_markup (GTK_LABEL (secondary_label), TRUE);
@@ -133,25 +164,49 @@ set_message_area_text_and_icon (GeditMessageArea *message_area,
 		gtk_misc_set_alignment (GTK_MISC (secondary_label), 0, 0.5);
 	}
 
-	gedit_message_area_set_contents (GEDIT_MESSAGE_AREA (message_area),
-					 hbox_content);
+	gtk_widget_show_all (hbox_content);
+	set_contents (message_area, hbox_content);
 }
 
 static GtkWidget *
 create_io_loading_error_message_area (const gchar *primary_text,
-				      const gchar *secondary_text)
+				      const gchar *secondary_text,
+				      gboolean     recoverable_error)
 {
 	GtkWidget *message_area;
 
+#if !GTK_CHECK_VERSION (2, 17, 1)
 	message_area = gedit_message_area_new_with_buttons (
 					GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 					NULL);
+#else
+	message_area = gtk_info_bar_new_with_buttons (
+					GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					NULL);
+	gtk_info_bar_set_message_type (GTK_INFO_BAR (message_area),
+				       GTK_MESSAGE_ERROR);
+#endif
 
-	set_message_area_text_and_icon (GEDIT_MESSAGE_AREA (message_area),
+	set_message_area_text_and_icon (message_area,
 					"gtk-dialog-error",
 					primary_text,
 					secondary_text);
 
+	if (recoverable_error)
+	{
+#if !GTK_CHECK_VERSION (2, 17, 1)
+		gedit_message_area_add_stock_button_with_text (GEDIT_MESSAGE_AREA (message_area),
+							       _("_Retry"),
+							       GTK_STOCK_REFRESH,
+							       GTK_RESPONSE_OK);
+#else
+		info_bar_add_stock_button_with_text (GTK_INFO_BAR (message_area),
+						     _("_Retry"),
+						     GTK_STOCK_REFRESH,
+						     GTK_RESPONSE_OK);
+#endif
+	}
+
 	return message_area;
 }
 
@@ -181,7 +236,7 @@ parse_gio_error (gint          code,
 
 			if ((scheme_string != NULL) && g_utf8_validate (scheme_string, -1, NULL))
 			{
-	  			scheme_markup = g_markup_printf_escaped ("<i>%s:</i>", scheme_string);
+				scheme_markup = g_markup_printf_escaped ("<i>%s:</i>", scheme_string);
 
 				/* Translators: %s is a URI scheme (like for example http:, ftp:, etc.) */
 				*message_details = g_strdup_printf (_("gedit cannot handle %s locations."),
@@ -232,18 +287,21 @@ parse_gio_error (gint          code,
 			{
 				if (hn != NULL)
 				{
-					gchar *host_name = gedit_utils_make_valid_utf8 (hn);
+					gchar *host_markup;
+					gchar *host_name;
+
+					host_name = gedit_utils_make_valid_utf8 (hn);
 					g_free (hn);
 
-					gchar *host_markup = g_markup_printf_escaped ("<i>%s</i>", host_name);
+					host_markup = g_markup_printf_escaped ("<i>%s</i>", host_name);
 					g_free (host_name);
 
 					/* Translators: %s is a host name */
 					*message_details = g_strdup_printf (
 						_("Host %s could not be found. "
-	        		  	  	"Please check that your proxy settings "
-				  	  	"are correct and try again."),
-					  	host_markup);
+						"Please check that your proxy settings "
+						"are correct and try again."),
+						host_markup);
 
 					g_free (host_markup);
 				}
@@ -268,12 +326,12 @@ parse_gio_error (gint          code,
 	case G_IO_ERROR_TIMED_OUT:
 		*message_details = g_strdup (_("Connection timed out. Please try again."));
 		break;
-	
+
 	default:
 		ret = FALSE;
 		break;
 	}
-	
+
 	return ret;
 }
 
@@ -285,18 +343,18 @@ parse_gedit_error (gint          code,
 	           const gchar  *uri_for_display)
 {
 	gboolean ret = TRUE;
-	
+
 	switch (code)
 	{
 	case GEDIT_DOCUMENT_ERROR_TOO_BIG:
 		*message_details = g_strdup (_("The file is too big."));
 		break;
-		
+
 	default:
 		ret = FALSE;
 		break;
 	}
-	
+
 	return ret;
 }
 
@@ -308,7 +366,7 @@ parse_error (const GError *error,
 	     const gchar  *uri_for_display)
 {
 	gboolean ret = FALSE;
-	
+
 	if (error->domain == G_IO_ERROR)
 	{
 		ret = parse_gio_error (error->code, 
@@ -321,9 +379,9 @@ parse_error (const GError *error,
 	{
 		ret = parse_gedit_error (error->code,
 					 error_message, 
-				         message_details,
-  				         uri,
-				         uri_for_display);
+					 message_details,
+					 uri,
+					 uri_for_display);
 	}
 	
 	if (!ret)
@@ -335,70 +393,6 @@ parse_error (const GError *error,
 	}
 }
 
-GtkWidget *
-gedit_io_loading_error_message_area_new (const gchar  *uri,
-					 const GError *error)
-{
-	gchar *error_message = NULL;
-	gchar *message_details = NULL;
-	gchar *full_formatted_uri;
-       	gchar *uri_for_display;
-       	gchar *temp_uri_for_display;
-	GtkWidget *message_area;
-
-	g_return_val_if_fail (uri != NULL, NULL);
-	g_return_val_if_fail (error != NULL, NULL);
-	g_return_val_if_fail ((error->domain == GEDIT_DOCUMENT_ERROR) || 
-			      (error->domain == G_IO_ERROR), NULL);
-
-	full_formatted_uri = gedit_utils_uri_for_display (uri);
-
-	/* Truncate the URI so it doesn't get insanely wide. Note that even
-	 * though the dialog uses wrapped text, if the URI doesn't contain
-	 * white space then the text-wrapping code is too stupid to wrap it.
-	 */
-	temp_uri_for_display = gedit_utils_str_middle_truncate (full_formatted_uri, 
-								MAX_URI_IN_DIALOG_LENGTH);								
-	g_free (full_formatted_uri);
-
-	uri_for_display = g_markup_printf_escaped ("<i>%s</i>", temp_uri_for_display);
-	g_free (temp_uri_for_display);
-
-	if (is_gio_error (error, G_IO_ERROR_TOO_MANY_LINKS))
-	{
-		message_details = g_strdup (_("The number of followed links is limited and the actual file could not be found within this limit."));
-	}
-	else if (is_gio_error (error, G_IO_ERROR_PERMISSION_DENIED))
-	{
-		message_details = g_strdup (_("You do not have the permissions necessary to open the file."));
-	}
-	else
-	{
-		parse_error (error, &error_message, &message_details, uri, uri_for_display);
-	}
-
-	if (error_message == NULL)
-		error_message = g_strdup_printf (_("Could not open the file %s."),
-						 uri_for_display);
-
-	message_area = create_io_loading_error_message_area (error_message,
-							     message_details);
-
-	if (is_recoverable_error (error))
-	{
-		gedit_message_area_add_stock_button_with_text (GEDIT_MESSAGE_AREA (message_area),
-							       _("_Retry"),
-							       GTK_STOCK_REFRESH,
-							       GTK_RESPONSE_OK);
-	}
-
-	g_free (uri_for_display);
-	g_free (error_message);
-	g_free (message_details);
-
-	return message_area;
-}
-
 GtkWidget *
 gedit_unrecoverable_reverting_error_message_area_new (const gchar  *uri,
 						      const GError *error)
@@ -406,8 +400,8 @@ gedit_unrecoverable_reverting_error_message_area_new (const gchar  *uri,
 	gchar *error_message = NULL;
 	gchar *message_details = NULL;
 	gchar *full_formatted_uri;
-       	gchar *uri_for_display;
-       	gchar *temp_uri_for_display;
+	gchar *uri_for_display;
+	gchar *temp_uri_for_display;
 	GtkWidget *message_area;
 
 	g_return_val_if_fail (uri != NULL, NULL);
@@ -443,7 +437,8 @@ gedit_unrecoverable_reverting_error_message_area_new (const gchar  *uri,
 						 uri_for_display);
 
 	message_area = create_io_loading_error_message_area (error_message,
-							     message_details);
+							     message_details,
+							     FALSE);
 
 	g_free (uri_for_display);
 	g_free (error_message);
@@ -453,13 +448,13 @@ gedit_unrecoverable_reverting_error_message_area_new (const gchar  *uri,
 }
 
 static void
-create_option_menu (GtkWidget *message_area, GtkWidget *vbox)
+create_combo_box (GtkWidget *message_area, GtkWidget *vbox)
 {
 	GtkWidget *hbox;
 	GtkWidget *label;
 	GtkWidget *menu;
 	gchar *label_markup;
-	
+
 	hbox = gtk_hbox_new (FALSE, 6);
 
 	label_markup = g_strdup_printf ("<small>%s</small>",
@@ -467,32 +462,32 @@ create_option_menu (GtkWidget *message_area, GtkWidget *vbox)
 	label = gtk_label_new_with_mnemonic (label_markup);
 	g_free (label_markup);
 	gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
-	menu = gedit_encodings_option_menu_new (TRUE);
+	menu = gedit_encodings_combo_box_new (TRUE);
 	g_object_set_data (G_OBJECT (message_area), 
 			   "gedit-message-area-encoding-menu", 
 			   menu);
-	
-	gtk_label_set_mnemonic_widget (GTK_LABEL (label), menu);				       		
-	gtk_box_pack_start (GTK_BOX (hbox), 
+
+	gtk_label_set_mnemonic_widget (GTK_LABEL (label), menu);
+	gtk_box_pack_start (GTK_BOX (hbox),
 			    label,
 			    FALSE,
 			    FALSE,
 			    0);
 
-	gtk_box_pack_end (GTK_BOX (hbox), 
-			  menu,
-			  TRUE,
-			  TRUE,
-			  0);
+	gtk_box_pack_start (GTK_BOX (hbox),
+			    menu,
+			    FALSE,
+			    FALSE,
+			    0);
 
 	gtk_widget_show_all (hbox);
-
 	gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
 }
 
 static GtkWidget *
 create_conversion_error_message_area (const gchar *primary_text,
-				      const gchar *secondary_text)
+				      const gchar *secondary_text,
+				      gboolean     edit_anyway)
 {
 	GtkWidget *message_area;
 	GtkWidget *hbox_content;
@@ -502,47 +497,88 @@ create_conversion_error_message_area (const gchar *primary_text,
 	gchar *secondary_markup;
 	GtkWidget *primary_label;
 	GtkWidget *secondary_label;
-	
+
+#if !GTK_CHECK_VERSION (2, 17, 1)
 	message_area = gedit_message_area_new ();
 
-	gedit_message_area_add_stock_button_with_text (GEDIT_MESSAGE_AREA (message_area),						        
+	gedit_message_area_add_stock_button_with_text (GEDIT_MESSAGE_AREA (message_area),
 						       _("_Retry"),
 						       GTK_STOCK_REDO,
 						       GTK_RESPONSE_OK);
-	gedit_message_area_add_button (GEDIT_MESSAGE_AREA (message_area),
-				       GTK_STOCK_CANCEL,
-				       GTK_RESPONSE_CANCEL);
-				      
+
+	if (edit_anyway)
+	{
+		gedit_message_area_add_button (GEDIT_MESSAGE_AREA (message_area),
+					       _("Edit Any_way"),
+					       GTK_RESPONSE_YES);
+		gedit_message_area_add_button (GEDIT_MESSAGE_AREA (message_area),
+					       _("D_on't Edit"),
+					       GTK_RESPONSE_NO);
+	}
+	else
+	{
+		gedit_message_area_add_button (GEDIT_MESSAGE_AREA (message_area),
+					       GTK_STOCK_CANCEL,
+					       GTK_RESPONSE_CANCEL);
+	}
+#else
+	message_area = gtk_info_bar_new ();
+
+	info_bar_add_stock_button_with_text (GTK_INFO_BAR (message_area),
+					     _("_Retry"),
+					     GTK_STOCK_REDO,
+					     GTK_RESPONSE_OK);
+
+	if (edit_anyway)
+	{
+		gtk_info_bar_add_button (GTK_INFO_BAR (message_area),
+		/* Translators: the access key chosen for this string should be
+		 different from other main menu access keys (Open, Edit, View...) */
+					 _("Edit Any_way"),
+					 GTK_RESPONSE_YES);
+		gtk_info_bar_add_button (GTK_INFO_BAR (message_area),
+		/* Translators: the access key chosen for this string should be
+		 different from other main menu access keys (Open, Edit, View...) */
+					 _("D_on't Edit"),
+					 GTK_RESPONSE_NO);
+		gtk_info_bar_set_message_type (GTK_INFO_BAR (message_area),
+					       GTK_MESSAGE_WARNING);
+	}
+	else
+	{
+		gtk_info_bar_add_button (GTK_INFO_BAR (message_area),
+					 GTK_STOCK_CANCEL,
+					 GTK_RESPONSE_CANCEL);
+		gtk_info_bar_set_message_type (GTK_INFO_BAR (message_area),
+					       GTK_MESSAGE_ERROR);
+	}
+#endif
+
 	hbox_content = gtk_hbox_new (FALSE, 8);
-	gtk_widget_show (hbox_content);				
-	
+
 	image = gtk_image_new_from_stock ("gtk-dialog-error", GTK_ICON_SIZE_DIALOG);
-	gtk_widget_show (image);
 	gtk_box_pack_start (GTK_BOX (hbox_content), image, FALSE, FALSE, 0);
 	gtk_misc_set_alignment (GTK_MISC (image), 0.5, 0);
   
 	vbox = gtk_vbox_new (FALSE, 6);
-	gtk_widget_show (vbox);
 	gtk_box_pack_start (GTK_BOX (hbox_content), vbox, TRUE, TRUE, 0);
 	
 	primary_markup = g_strdup_printf ("<b>%s</b>", primary_text);
 	primary_label = gtk_label_new (primary_markup);
 	g_free (primary_markup);
-	gtk_widget_show (primary_label);
 	gtk_box_pack_start (GTK_BOX (vbox), primary_label, TRUE, TRUE, 0);
 	gtk_label_set_use_markup (GTK_LABEL (primary_label), TRUE);
 	gtk_label_set_line_wrap (GTK_LABEL (primary_label), TRUE);
 	gtk_misc_set_alignment (GTK_MISC (primary_label), 0, 0.5);
 	GTK_WIDGET_SET_FLAGS (primary_label, GTK_CAN_FOCUS);
 	gtk_label_set_selectable (GTK_LABEL (primary_label), TRUE);
-  
-  	if (secondary_text != NULL)
-  	{
-  		secondary_markup = g_strdup_printf ("<small>%s</small>",
-  						    secondary_text);
+
+	if (secondary_text != NULL)
+	{
+		secondary_markup = g_strdup_printf ("<small>%s</small>",
+						    secondary_text);
 		secondary_label = gtk_label_new (secondary_markup);
 		g_free (secondary_markup);
-		gtk_widget_show (secondary_label);
 		gtk_box_pack_start (GTK_BOX (vbox), secondary_label, TRUE, TRUE, 0);
 		GTK_WIDGET_SET_FLAGS (secondary_label, GTK_CAN_FOCUS);
 		gtk_label_set_use_markup (GTK_LABEL (secondary_label), TRUE);
@@ -551,31 +587,34 @@ create_conversion_error_message_area (const gchar *primary_text,
 		gtk_misc_set_alignment (GTK_MISC (secondary_label), 0, 0.5);
 	}
 
-	create_option_menu (message_area, vbox);
-	
-	gedit_message_area_set_contents (GEDIT_MESSAGE_AREA (message_area),
-					 hbox_content);
+	create_combo_box (message_area, vbox);
+	gtk_widget_show_all (hbox_content);
+	set_contents (message_area, hbox_content);
+
 	return message_area;
 }
 
 GtkWidget *
-gedit_conversion_error_while_loading_message_area_new (
-						const gchar         *uri,
-						const GeditEncoding *encoding,
-				    		const GError        *error)
+gedit_io_loading_error_message_area_new (const gchar         *uri,
+					 const GeditEncoding *encoding,
+					 const GError        *error)
 {
 	gchar *error_message = NULL;
 	gchar *message_details = NULL;
 	gchar *full_formatted_uri;
 	gchar *encoding_name;
-       	gchar *uri_for_display;
-       	gchar *temp_uri_for_display;
+	gchar *uri_for_display;
+	gchar *temp_uri_for_display;
 	GtkWidget *message_area;
+	gboolean edit_anyway = FALSE;
+	gboolean convert_error = FALSE;
 	
 	g_return_val_if_fail (uri != NULL, NULL);
 	g_return_val_if_fail (error != NULL, NULL);
 	g_return_val_if_fail ((error->domain == G_CONVERT_ERROR) ||
-			      (error->domain == GEDIT_CONVERT_ERROR), NULL);
+			      (error->domain == GEDIT_CONVERT_ERROR) ||
+			      (error->domain == GEDIT_DOCUMENT_ERROR) || 
+			      (error->domain == G_IO_ERROR), NULL);
 	
 	full_formatted_uri = gedit_utils_uri_for_display (uri);
 
@@ -583,8 +622,8 @@ gedit_conversion_error_while_loading_message_area_new (
 	 * though the dialog uses wrapped text, if the URI doesn't contain
 	 * white space then the text-wrapping code is too stupid to wrap it.
 	 */
-	temp_uri_for_display = gedit_utils_str_middle_truncate (full_formatted_uri, 
-								MAX_URI_IN_DIALOG_LENGTH);								
+	temp_uri_for_display = gedit_utils_str_middle_truncate (full_formatted_uri,
+								MAX_URI_IN_DIALOG_LENGTH);
 	g_free (full_formatted_uri);
 	
 	uri_for_display = g_markup_printf_escaped ("<i>%s</i>", temp_uri_for_display);
@@ -595,28 +634,65 @@ gedit_conversion_error_while_loading_message_area_new (
 	else
 		encoding_name = g_strdup ("UTF-8");
 
-	if (error->domain == GEDIT_CONVERT_ERROR)
+	if (is_gio_error (error, G_IO_ERROR_TOO_MANY_LINKS))
+	{
+		message_details = g_strdup (_("The number of followed links is limited and the actual file could not be found within this limit."));
+	}
+	else if (is_gio_error (error, G_IO_ERROR_PERMISSION_DENIED))
+	{
+		message_details = g_strdup (_("You do not have the permissions necessary to open the file."));
+	}
+	else if ((is_gio_error (error, G_IO_ERROR_INVALID_DATA) && encoding == NULL) ||
+	         (error->domain == GEDIT_CONVERT_ERROR &&
+	         error->code == GEDIT_CONVERT_ERROR_AUTO_DETECTION_FAILED))
 	{
-		g_return_val_if_fail (error->code == GEDIT_CONVERT_ERROR_AUTO_DETECTION_FAILED, NULL);
-		
 		error_message = g_strdup_printf (_("Could not open the file %s."),
-							 uri_for_display);
+						 uri_for_display);
 		message_details = g_strconcat (_("gedit has not been able to detect "
 				               "the character coding."), "\n", 
 				               _("Please check that you are not trying to open a binary file."), "\n",
 					       _("Select a character coding from the menu and try again."), NULL);
+		convert_error = TRUE;
 	}
-	else 
+	else if (error->domain == GEDIT_DOCUMENT_ERROR &&
+	         error->code == GEDIT_DOCUMENT_ERROR_CONVERSION_FALLBACK)
+	{
+		error_message = g_strdup_printf (_("There was a problem opening the file %s."),
+						 uri_for_display);
+		message_details = g_strconcat (_("The file you opened has some invalid characters, "
+					       "if you continue editing this file you could make this "
+					       "document useless."), "\n",
+					       _("You can also choose another character encoding and try again."),
+					       NULL);
+		edit_anyway = TRUE;
+		convert_error = TRUE;
+	}
+	else if (is_gio_error (error, G_IO_ERROR_INVALID_DATA) && encoding != NULL)
 	{
-		
 		error_message = g_strdup_printf (_("Could not open the file %s using the %s character coding."),
 						 uri_for_display, 
 						 encoding_name);
 		message_details = g_strconcat (_("Please check that you are not trying to open a binary file."), "\n",
 					       _("Select a different character coding from the menu and try again."), NULL);
+		convert_error = TRUE;
+	}
+	else
+	{
+		parse_error (error, &error_message, &message_details, uri, uri_for_display);
+	}
+
+	if (convert_error)
+	{
+		message_area = create_conversion_error_message_area (error_message,
+								     message_details,
+								     edit_anyway);
+	}
+	else
+	{
+		message_area = create_io_loading_error_message_area (error_message,
+								     message_details,
+								     is_recoverable_error (error));
 	}
-	
-	message_area = create_conversion_error_message_area (error_message, message_details);
 
 	g_free (uri_for_display);
 	g_free (encoding_name);
@@ -636,8 +712,8 @@ gedit_conversion_error_while_saving_message_area_new (
 	gchar *message_details = NULL;
 	gchar *full_formatted_uri;
 	gchar *encoding_name;
-       	gchar *uri_for_display;
-       	gchar *temp_uri_for_display;
+	gchar *uri_for_display;
+	gchar *temp_uri_for_display;
 	GtkWidget *message_area;
 	
 	g_return_val_if_fail (uri != NULL, NULL);
@@ -669,7 +745,8 @@ gedit_conversion_error_while_saving_message_area_new (
 	
 	message_area = create_conversion_error_message_area (
 								error_message,
-								message_details);
+								message_details,
+								FALSE);
 
 	g_free (uri_for_display);
 	g_free (encoding_name);
@@ -683,15 +760,19 @@ const GeditEncoding *
 gedit_conversion_error_message_area_get_encoding (GtkWidget *message_area)
 {
 	gpointer menu;
-	
+
+#if !GTK_CHECK_VERSION (2, 17, 1)
 	g_return_val_if_fail (GEDIT_IS_MESSAGE_AREA (message_area), NULL);
+#else
+	g_return_val_if_fail (GTK_IS_INFO_BAR (message_area), NULL);
+#endif
 
 	menu = g_object_get_data (G_OBJECT (message_area), 
 				  "gedit-message-area-encoding-menu");	
 	g_return_val_if_fail (menu, NULL);
 	
-	return gedit_encodings_option_menu_get_selected_encoding
-					(GEDIT_ENCODINGS_OPTION_MENU (menu));
+	return gedit_encodings_combo_box_get_selected_encoding
+					(GEDIT_ENCODINGS_COMBO_BOX (menu));
 }
 
 GtkWidget *
@@ -708,8 +789,8 @@ gedit_file_already_open_warning_message_area_new (const gchar *uri)
 	gchar *primary_text;
 	const gchar *secondary_text;
 	gchar *full_formatted_uri;
-       	gchar *uri_for_display;
-       	gchar *temp_uri_for_display;
+	gchar *uri_for_display;
+	gchar *temp_uri_for_display;
 	
 	full_formatted_uri = gedit_utils_uri_for_display (uri);
 
@@ -723,25 +804,38 @@ gedit_file_already_open_warning_message_area_new (const gchar *uri)
 	
 	uri_for_display = g_markup_printf_escaped ("<i>%s</i>", temp_uri_for_display);
 	g_free (temp_uri_for_display);
-	
+
+#if !GTK_CHECK_VERSION (2, 17, 1)
 	message_area = gedit_message_area_new ();
 	gedit_message_area_add_button (GEDIT_MESSAGE_AREA (message_area),
-				       _("_Edit Anyway"),
+				       _("Edit Any_way"),
 				       GTK_RESPONSE_YES);
 	gedit_message_area_add_button (GEDIT_MESSAGE_AREA (message_area),
-				       _("_Don't Edit"),
+				       _("D_on't Edit"),
 				       GTK_RESPONSE_CANCEL);
-				       
+#else
+	message_area = gtk_info_bar_new ();
+	gtk_info_bar_add_button (GTK_INFO_BAR (message_area),
+	/* Translators: the access key chosen for this string should be
+	 different from other main menu access keys (Open, Edit, View...) */
+				 _("Edit Any_way"),
+				 GTK_RESPONSE_YES);
+	gtk_info_bar_add_button (GTK_INFO_BAR (message_area),
+	/* Translators: the access key chosen for this string should be
+	 different from other main menu access keys (Open, Edit, View...) */
+				 _("D_on't Edit"),
+				 GTK_RESPONSE_CANCEL);
+	gtk_info_bar_set_message_type (GTK_INFO_BAR (message_area),
+				       GTK_MESSAGE_WARNING);
+#endif
+
 	hbox_content = gtk_hbox_new (FALSE, 8);
-	gtk_widget_show (hbox_content);				
-	
+
 	image = gtk_image_new_from_stock ("gtk-dialog-warning", GTK_ICON_SIZE_DIALOG);
-	gtk_widget_show (image);
 	gtk_box_pack_start (GTK_BOX (hbox_content), image, FALSE, FALSE, 0);
 	gtk_misc_set_alignment (GTK_MISC (image), 0.5, 0);
-  
+
 	vbox = gtk_vbox_new (FALSE, 6);
-	gtk_widget_show (vbox);
 	gtk_box_pack_start (GTK_BOX (hbox_content), vbox, TRUE, TRUE, 0);
 
 	primary_text = g_strdup_printf (_("This file (%s) is already open in another gedit window."), uri_for_display);
@@ -751,7 +845,6 @@ gedit_file_already_open_warning_message_area_new (const gchar *uri)
 	g_free (primary_text);
 	primary_label = gtk_label_new (primary_markup);
 	g_free (primary_markup);
-	gtk_widget_show (primary_label);
 	gtk_box_pack_start (GTK_BOX (vbox), primary_label, TRUE, TRUE, 0);
 	gtk_label_set_use_markup (GTK_LABEL (primary_label), TRUE);
 	gtk_label_set_line_wrap (GTK_LABEL (primary_label), TRUE);
@@ -761,11 +854,10 @@ gedit_file_already_open_warning_message_area_new (const gchar *uri)
 
 	secondary_text = _("gedit opened this instance of the file in a non-editable way. "
 			   "Do you want to edit it anyway?");
-  	secondary_markup = g_strdup_printf ("<small>%s</small>",
-  					    secondary_text);
+	secondary_markup = g_strdup_printf ("<small>%s</small>",
+					    secondary_text);
 	secondary_label = gtk_label_new (secondary_markup);
 	g_free (secondary_markup);
-	gtk_widget_show (secondary_label);
 	gtk_box_pack_start (GTK_BOX (vbox), secondary_label, TRUE, TRUE, 0);
 	GTK_WIDGET_SET_FLAGS (secondary_label, GTK_CAN_FOCUS);
 	gtk_label_set_use_markup (GTK_LABEL (secondary_label), TRUE);
@@ -773,8 +865,9 @@ gedit_file_already_open_warning_message_area_new (const gchar *uri)
 	gtk_label_set_selectable (GTK_LABEL (secondary_label), TRUE);
 	gtk_misc_set_alignment (GTK_MISC (secondary_label), 0, 0.5);
 
-	gedit_message_area_set_contents (GEDIT_MESSAGE_AREA (message_area),
-					 hbox_content);
+	gtk_widget_show_all (hbox_content);
+	set_contents (message_area, hbox_content);
+
 	return message_area;
 }
 
@@ -815,6 +908,7 @@ gedit_externally_modified_saving_error_message_area_new (
 	uri_for_display = g_markup_printf_escaped ("<i>%s</i>", temp_uri_for_display);
 	g_free (temp_uri_for_display);
 
+#if !GTK_CHECK_VERSION (2, 17, 1)
 	message_area = gedit_message_area_new ();
 	gedit_message_area_add_stock_button_with_text (GEDIT_MESSAGE_AREA (message_area),
 						       _("S_ave Anyway"),
@@ -823,17 +917,27 @@ gedit_externally_modified_saving_error_message_area_new (
 	gedit_message_area_add_button (GEDIT_MESSAGE_AREA (message_area),
 				       _("D_on't Save"),
 				       GTK_RESPONSE_CANCEL);
+#else
+	message_area = gtk_info_bar_new ();
+	
+	info_bar_add_stock_button_with_text (GTK_INFO_BAR (message_area),
+					     _("S_ave Anyway"),
+					     GTK_STOCK_SAVE,
+					     GTK_RESPONSE_YES);
+	gtk_info_bar_add_button (GTK_INFO_BAR (message_area),
+				 _("D_on't Save"),
+				 GTK_RESPONSE_CANCEL);
+	gtk_info_bar_set_message_type (GTK_INFO_BAR (message_area),
+				       GTK_MESSAGE_WARNING);
+#endif
 
 	hbox_content = gtk_hbox_new (FALSE, 8);
-	gtk_widget_show (hbox_content);				
 
 	image = gtk_image_new_from_stock ("gtk-dialog-warning", GTK_ICON_SIZE_DIALOG);
-	gtk_widget_show (image);
 	gtk_box_pack_start (GTK_BOX (hbox_content), image, FALSE, FALSE, 0);
 	gtk_misc_set_alignment (GTK_MISC (image), 0.5, 0);
 
 	vbox = gtk_vbox_new (FALSE, 6);
-	gtk_widget_show (vbox);
 	gtk_box_pack_start (GTK_BOX (hbox_content), vbox, TRUE, TRUE, 0);
 
 	// FIXME: review this message, it's not clear since for the user the "modification"
@@ -847,7 +951,6 @@ gedit_externally_modified_saving_error_message_area_new (
 	g_free (primary_text);
 	primary_label = gtk_label_new (primary_markup);
 	g_free (primary_markup);
-	gtk_widget_show (primary_label);
 	gtk_box_pack_start (GTK_BOX (vbox), primary_label, TRUE, TRUE, 0);
 	gtk_label_set_use_markup (GTK_LABEL (primary_label), TRUE);
 	gtk_label_set_line_wrap (GTK_LABEL (primary_label), TRUE);
@@ -856,11 +959,10 @@ gedit_externally_modified_saving_error_message_area_new (
 	gtk_label_set_selectable (GTK_LABEL (primary_label), TRUE);
 
 	secondary_text = _("If you save it, all the external changes could be lost. Save it anyway?");
-  	secondary_markup = g_strdup_printf ("<small>%s</small>",
+	secondary_markup = g_strdup_printf ("<small>%s</small>",
 					    secondary_text);
 	secondary_label = gtk_label_new (secondary_markup);
 	g_free (secondary_markup);
-	gtk_widget_show (secondary_label);
 	gtk_box_pack_start (GTK_BOX (vbox), secondary_label, TRUE, TRUE, 0);
 	GTK_WIDGET_SET_FLAGS (secondary_label, GTK_CAN_FOCUS);
 	gtk_label_set_use_markup (GTK_LABEL (secondary_label), TRUE);
@@ -868,8 +970,8 @@ gedit_externally_modified_saving_error_message_area_new (
 	gtk_label_set_selectable (GTK_LABEL (secondary_label), TRUE);
 	gtk_misc_set_alignment (GTK_MISC (secondary_label), 0, 0.5);
 
-	gedit_message_area_set_contents (GEDIT_MESSAGE_AREA (message_area),
-					 hbox_content);
+	gtk_widget_show_all (hbox_content);
+	set_contents (message_area, hbox_content);
 
 	return message_area;
 }
@@ -912,6 +1014,7 @@ gedit_no_backup_saving_error_message_area_new (const gchar  *uri,
 	uri_for_display = g_markup_printf_escaped ("<i>%s</i>", temp_uri_for_display);
 	g_free (temp_uri_for_display);
 
+#if !GTK_CHECK_VERSION (2, 17, 1)
 	message_area = gedit_message_area_new ();
 	gedit_message_area_add_stock_button_with_text (GEDIT_MESSAGE_AREA (message_area),
 						       _("S_ave Anyway"),
@@ -920,17 +1023,27 @@ gedit_no_backup_saving_error_message_area_new (const gchar  *uri,
 	gedit_message_area_add_button (GEDIT_MESSAGE_AREA (message_area),
 				       _("D_on't Save"),
 				       GTK_RESPONSE_CANCEL);
+#else
+	message_area = gtk_info_bar_new ();
+	
+	info_bar_add_stock_button_with_text (GTK_INFO_BAR (message_area),
+					     _("S_ave Anyway"),
+					     GTK_STOCK_SAVE,
+					     GTK_RESPONSE_YES);
+	gtk_info_bar_add_button (GTK_INFO_BAR (message_area),
+				 _("D_on't Save"),
+				 GTK_RESPONSE_CANCEL);
+	gtk_info_bar_set_message_type (GTK_INFO_BAR (message_area),
+				       GTK_MESSAGE_WARNING);
+#endif
 
 	hbox_content = gtk_hbox_new (FALSE, 8);
-	gtk_widget_show (hbox_content);
 
 	image = gtk_image_new_from_stock ("gtk-dialog-warning", GTK_ICON_SIZE_DIALOG);
-	gtk_widget_show (image);
 	gtk_box_pack_start (GTK_BOX (hbox_content), image, FALSE, FALSE, 0);
 	gtk_misc_set_alignment (GTK_MISC (image), 0.5, 0);
 
 	vbox = gtk_vbox_new (FALSE, 6);
-	gtk_widget_show (vbox);
 	gtk_box_pack_start (GTK_BOX (hbox_content), vbox, TRUE, TRUE, 0);
 
 	// FIXME: review this messages
@@ -948,7 +1061,6 @@ gedit_no_backup_saving_error_message_area_new (const gchar  *uri,
 	g_free (primary_text);
 	primary_label = gtk_label_new (primary_markup);
 	g_free (primary_markup);
-	gtk_widget_show (primary_label);
 	gtk_box_pack_start (GTK_BOX (vbox), primary_label, TRUE, TRUE, 0);
 	gtk_label_set_use_markup (GTK_LABEL (primary_label), TRUE);
 	gtk_label_set_line_wrap (GTK_LABEL (primary_label), TRUE);
@@ -959,11 +1071,10 @@ gedit_no_backup_saving_error_message_area_new (const gchar  *uri,
 	secondary_text = _("gedit could not backup the old copy of the file before saving the new one. "
 			   "You can ignore this warning and save the file anyway, but if an error "
 			   "occurs while saving, you could lose the old copy of the file. Save anyway?");
-  	secondary_markup = g_strdup_printf ("<small>%s</small>",
+	secondary_markup = g_strdup_printf ("<small>%s</small>",
 					    secondary_text);
 	secondary_label = gtk_label_new (secondary_markup);
 	g_free (secondary_markup);
-	gtk_widget_show (secondary_label);
 	gtk_box_pack_start (GTK_BOX (vbox), secondary_label, TRUE, TRUE, 0);
 	GTK_WIDGET_SET_FLAGS (secondary_label, GTK_CAN_FOCUS);
 	gtk_label_set_use_markup (GTK_LABEL (secondary_label), TRUE);
@@ -971,8 +1082,8 @@ gedit_no_backup_saving_error_message_area_new (const gchar  *uri,
 	gtk_label_set_selectable (GTK_LABEL (secondary_label), TRUE);
 	gtk_misc_set_alignment (GTK_MISC (secondary_label), 0, 0.5);
 
-	gedit_message_area_set_contents (GEDIT_MESSAGE_AREA (message_area),
-					 hbox_content);
+	gtk_widget_show_all (hbox_content);
+	set_contents (message_area, hbox_content);
 
 	return message_area;
 }
@@ -986,8 +1097,8 @@ gedit_unrecoverable_saving_error_message_area_new (const gchar  *uri,
 	gchar *full_formatted_uri;
 	gchar *scheme_string;
 	gchar *scheme_markup;
-       	gchar *uri_for_display;
-       	gchar *temp_uri_for_display;
+	gchar *uri_for_display;
+	gchar *temp_uri_for_display;
 	GtkWidget *message_area;
 
 	g_return_val_if_fail (uri != NULL, NULL);
@@ -1002,7 +1113,7 @@ gedit_unrecoverable_saving_error_message_area_new (const gchar  *uri,
 	 * white space then the text-wrapping code is too stupid to wrap it.
 	 */
 	temp_uri_for_display = gedit_utils_str_middle_truncate (full_formatted_uri, 
-								MAX_URI_IN_DIALOG_LENGTH);								
+								MAX_URI_IN_DIALOG_LENGTH);
 	g_free (full_formatted_uri);
 
 	uri_for_display = g_markup_printf_escaped ("<i>%s</i>", temp_uri_for_display);
@@ -1088,7 +1199,8 @@ gedit_unrecoverable_saving_error_message_area_new (const gchar  *uri,
 						 uri_for_display);
 
 	message_area = create_io_loading_error_message_area (error_message,
-							     message_details);
+							     message_details,
+							     FALSE);
 
 	g_free (uri_for_display);
 	g_free (error_message);
@@ -1101,9 +1213,9 @@ GtkWidget *
 gedit_externally_modified_message_area_new (const gchar *uri,
 					    gboolean     document_modified)
 {
-       	gchar *full_formatted_uri;
-       	gchar *uri_for_display;
-       	gchar *temp_uri_for_display;
+	gchar *full_formatted_uri;
+	gchar *uri_for_display;
+	gchar *temp_uri_for_display;
 	const gchar *primary_text;
 	const gchar *secondary_text;
 	GtkWidget *message_area;
@@ -1117,7 +1229,7 @@ gedit_externally_modified_message_area_new (const gchar *uri,
 	 * white space then the text-wrapping code is too stupid to wrap it.
 	 */
 	temp_uri_for_display = gedit_utils_str_middle_truncate (full_formatted_uri, 
-								MAX_URI_IN_DIALOG_LENGTH);								
+								MAX_URI_IN_DIALOG_LENGTH);
 	g_free (full_formatted_uri);
 
 	uri_for_display = g_markup_printf_escaped ("<i>%s</i>", temp_uri_for_display);
@@ -1135,13 +1247,9 @@ gedit_externally_modified_message_area_new (const gchar *uri,
 	else
 		secondary_text = _("Do you want to reload the file?");
 
+#if !GTK_CHECK_VERSION (2, 17, 1)
 	message_area = gedit_message_area_new ();
-
-	set_message_area_text_and_icon (GEDIT_MESSAGE_AREA (message_area),
-					"gtk-dialog-warning",
-					primary_text,
-					secondary_text);
-
+	
 	gedit_message_area_add_stock_button_with_text (GEDIT_MESSAGE_AREA (message_area),
 						       _("_Reload"),
 						       GTK_STOCK_REFRESH,
@@ -1150,6 +1258,24 @@ gedit_externally_modified_message_area_new (const gchar *uri,
 	gedit_message_area_add_button (GEDIT_MESSAGE_AREA (message_area),
 				       GTK_STOCK_CANCEL,
 				       GTK_RESPONSE_CANCEL);
+#else
+	message_area = gtk_info_bar_new ();
+	
+	info_bar_add_stock_button_with_text (GTK_INFO_BAR (message_area),
+					     _("_Reload"),
+					     GTK_STOCK_REFRESH,
+					     GTK_RESPONSE_OK);
+	gtk_info_bar_add_button (GTK_INFO_BAR (message_area),
+				 GTK_STOCK_CANCEL,
+				 GTK_RESPONSE_CANCEL);
+	gtk_info_bar_set_message_type (GTK_INFO_BAR (message_area),
+				       GTK_MESSAGE_WARNING);
+#endif
+
+	set_message_area_text_and_icon (message_area,
+					"gtk-dialog-warning",
+					primary_text,
+					secondary_text);
 
 	return message_area;
 }
